from pywong.db.mysql.utils import DbHelper as mysql_DbHelper
from pywong.db.mysql.settings import DEFAULT as mysql_DEFAULT, DATABASES as mysql_DATABASES, OPTION as mysql_OPTION

from pywong.db.postgresql.utils import DbHelper as pg_DbHelper
from pywong.db.postgresql.settings import DEFAULT as pg_DEFAULT, DATABASES as pg_DATABASES, OPTION as pg_OPTION

from pywong.db.mssql.utils import DbHelper as mssql_DbHelper
from pywong.db.mysql.settings import DEFAULT as mssql_DEFAULT, DATABASES as mssql_DATABASES, OPTION as mssql_OPTION

from pywong.db.settings import DATABASE_TYPE


# 获取数据库帮助类实例
def get_db_helper(db_type):
    mysql_helper = None
    pg_helper = None
    mssql_helper = None

    if (db_type == DATABASE_TYPE.MYSQL) or (db_type == DATABASE_TYPE.ALL):
        mysql_helper = mysql_DbHelper(host=mysql_DEFAULT['host'], user=mysql_DEFAULT['user'],
                                      password=mysql_DEFAULT['password'], port=mysql_DEFAULT['port'],
                                      database=mysql_DATABASES['default_database'])

    if (db_type == DATABASE_TYPE.POSTGRESQL) or (db_type == DATABASE_TYPE.ALL):
        pg_helper = pg_DbHelper(host=pg_DEFAULT['host'], user=pg_DEFAULT['user'],
                                password=pg_DEFAULT['password'], port=pg_DEFAULT['port'],
                                database=pg_DATABASES['default_database'])

    if (db_type == DATABASE_TYPE.SQLSERVER) or (db_type == DATABASE_TYPE.ALL):
        mssql_helper = mssql_DbHelper(server=mssql_DEFAULT['server'], user=mssql_DEFAULT['user'],
                                      password=mssql_DEFAULT['password'], port=mssql_DEFAULT['port'],
                                      database=mssql_DATABASES['default_database'])

    return mysql_helper, pg_helper, mssql_helper



from prophet import Prophet
import pandas as pd
import joblib
import os
from pywong.db.postgresql.utils import DbHelper
import warnings
import platform

# 忽略警告
warnings.filterwarnings('ignore')

"""
工作原理：
根据时间预测结果
"""


class TimeSeriesProphet:
    """时间序列"""

    def __init__(self, name, path='D:\\model'):
        """构造函数"""
        self.future = None
        self.forecast = None
        self.data = None
        # 模型名称
        self.name = name
        self.model_path = self.get_model_path(path)
        self.model = Prophet()

    def fit(self, path=None, sql=None, data=None):
        """拟合数据"""
        data_all = None
        if path is not None:
            data_all = pd.read_csv(path)

        if sql is not None:
            data_all = DbHelper.execute_query_pd(sql)

        if data is not None:
            data_all = data

        self.data = data_all

        self.model.fit(self.data)

        return self

    def make_future(self, periods, freq, include_history):
        """构造预测数据"""
        future = self.model.make_future_dataframe(periods=periods, freq=freq, include_history=include_history)
        self.future = future

    def predict(self, periods, freq, include_history=False):
        """预测"""
        self.make_future(periods, freq, include_history)
        forecast = self.model.predict(self.future)
        self.forecast = forecast

        return forecast

    def get_model_path(self, path):
        """返回模型路径"""
        file = None
        if not os.path.exists(path):
            os.mkdir(path)

        if 'Windows' == platform.system():
            file = f"{path}\\{self.name}.model"

        if 'Linux' == platform.system():
            file = f"{path}/{self.name}.model"

        return os.path.abspath(file)

    def save(self):
        """保存模型"""
        joblib.dump(self.model, self.model_path)
        print(f'保存{self.model_path}.')

    def load(self):
        """加载模型"""
        self.model = joblib.load(self.model_path)
        print(f"加载{self.name}.model.")

    def __str__(self):
        return "TimeSeriesProphet()"



from pyod.models.cof import COF


class AbnormalCOF:
    """异常值检测"""

    def __init__(self, data):
        """构造函数"""
        self.data = data

    def fit_predict(self):
        """异常值检测"""
        cof = COF()
        col = self.data.columns
        cof_label = cof.fit_predict(self.data)
        # 标签为1的为异常值
        self.data['label'] = cof_label
        # 获取处理后的正常数据
        self.data = self.data[self.data.label != 1][col]
        return self.data



import pymssql
from pywong.db.mssql.settings import OPTION


class DbHelper:
    """MsSQL工具类"""

    def __init__(self, server="127.0.0.1", user=None, password=None, database=None, port=1433, charset='utf8', autocommit=True, as_dict=True):
        """构造函数"""
        self.database = database
        self.conn = pymssql.connect(
            server=server,
            port=str(port),
            user=user,
            password=password,
            database=database,
            charset=charset,
            autocommit=autocommit,
            as_dict=as_dict
        )
        self.cursor = self.conn.cursor()

    def __del__(self):
        """析构函数"""
        self.conn.commit()
        self.cursor.close()
        self.conn.close()

    def get_conn(self):
        """获取连接"""
        return self.conn

    def get_cursor(self):
        """获取游标"""
        return self.cursor

    def execute(self, sql, option=OPTION.select, d_list=None):
        """获取SQL执行结果：含增删改查"""
        if option == OPTION.select:
            self.cursor.execute(sql)
            return self.cursor.fetchall()
        elif option == OPTION.insertMany:
            return self.cursor.executemany(sql, d_list)
        else:
            return self.cursor.execute(sql)



import psycopg2
from pywong.db.postgresql.settings import OPTION, DEFAULT, DATABASES
import pandas as pd
from sqlalchemy import create_engine


class DbHelper:
    """PostgreSQL工具类"""

    def __init__(self, host="127.0.0.1", user=None, password=None, database=None, port=5432,
                 dsn=None, connection_factory=None, cursor_factory=None, **kwargs):
        """构造函数"""
        self.database = database
        self.conn = psycopg2.connect(host=host, user=user, password=password, database=database, port=str(port),
                                     dsn=dsn, connection_factory=connection_factory, cursor_factory=cursor_factory,
                                     **kwargs)
        self.cursor = self.conn.cursor()

    def __del__(self):
        """析构函数"""
        self.conn.commit()
        self.cursor.close()
        self.conn.close()

    def get_conn(self):
        """获取连接"""
        return self.conn

    def get_cursor(self):
        """获取游标"""
        return self.cursor

    def execute(self, sql, option=OPTION.select, d_list=None, vars=None):
        """获取SQL执行结果：含增删改查"""
        if option == OPTION.select:
            self.cursor.execute(sql, vars)
            return self.cursor.fetchall()
        elif option == OPTION.insertMany:
            return self.cursor.executemany(sql, d_list)
        else:
            return self.cursor.execute(sql, vars)

    @staticmethod
    def execute_query_pd(sql):
        """获取SQL查询结果，pandas方式"""
        return pd.read_sql(sql, create_engine(f"postgresql://{DEFAULT['user']}:{DEFAULT['password']}@{DEFAULT['host']}:"
                                              f"{DEFAULT['port']}/{DATABASES['default_database']}"))



"""
资源监测
"""
import psutil


class Resource:
    """资源类"""
    def __init__(self):
        """构造函数"""
        self.memory = Memory()
        self.cpu = CPU()
        self.hard_disk = HardDisk()

    def get_all(self):
        """获取所有信息"""
        self.get_memory()
        self.get_cpu()
        self.get_hard_disk()

    def get_memory(self):
        """获取内存信息"""
        self.memory.get_memory_info()

    def get_cpu(self):
        """获取CPU信息"""
        self.cpu.get_cpu_nfo()

    def get_hard_disk(self):
        """获取磁盘信息"""
        self.hard_disk.get_hard_disk_info()


class Memory:
    """内存类"""
    def __init__(self):
        """构造函数"""
        self.total = None
        self.used = None
        self.free = None
        self.available = None
        self.percent = None

        self.get_memory_info()

    def get_memory_info(self):
        """获取内存信息"""
        memory = psutil.virtual_memory()
        self.total = memory.total
        self.used = memory.used
        self.free = memory.free
        self.available = memory.available
        self.percent = memory.percent

    def __str__(self):
        return "Memory usage rate: {}%".format(self.percent)

    def __repr__(self):
        return self.__str__()


class CPU:
    """CPU类"""
    def __init__(self):
        """构造函数"""
        self.logical_count = None
        self.physical_count = None
        self.usage_rate = None
        self.user_time = None
        self.io_wait_time = None

        self.get_cpu_nfo()

    def get_cpu_nfo(self):
        """获取CPU信息"""
        self.logical_count = psutil.cpu_count(logical=True),
        self.physical_count = psutil.cpu_count(logical=False),
        self.usage_rate = psutil.cpu_percent(),
        self.user_time = psutil.cpu_times().user,

    def __str__(self):
        return "CPU usage rate: {}%".format(self.usage_rate[0])

    def __repr__(self):
        return self.__str__()


class HardDisk:
    """磁盘类"""
    def __init__(self):
        """构造函数"""
        self.disk_partition = None
        self.disk_usage = None
        self.io_counters = None

        self.get_hard_disk_info()

    def get_hard_disk_info(self):
        """获取磁盘信息"""
        self.disk_partition = psutil.disk_partitions()
        self.disk_usage = [psutil.disk_usage(d.device) for d in self.disk_partition]
        self.io_counters = psutil.disk_io_counters()

    @staticmethod
    def get_hard_disk_usage(path):
        """获取磁盘使用率"""
        # noinspection PyBroadException
        try:
            return psutil.disk_usage(path)
        except Exception as e:
            return e.__str__()

    @staticmethod
    def get_io_counters(disk=True):
        """获取IO数"""
        return psutil.disk_io_counters(perdisk=disk)

    def __str__(self):
        res = ""
        try:
            for d in range(len(self.disk_partition)):
                item_str = "HardDisk {} usage rate: {}%".format(self.disk_partition[d].device,
                                                                self.disk_usage[d].percent)
                res = res + item_str + "\n"
            return res
        except Exception as e:
            return e.__str__()

    def __repr__(self):
        return self.__str__()


import pymysql.cursors
from pywong.db.mysql.settings import OPTION, DEFAULT, DATABASES
import pandas as pd
from sqlalchemy import create_engine


class DbHelper:
    """MySQL工具类"""

    def __init__(self, host="127.0.0.1", user=None, password=None, database=None, port=3306, charset='utf8'):
        """构造函数"""
        self.database = database
        self.conn = pymysql.connect(
            host=host,
            port=port,
            user=user,
            password=password,
            database=database,
            charset=charset,
            cursorclass=pymysql.cursors.DictCursor
        )
        self.cursor = self.conn.cursor()

    def __del__(self):
        """析构函数"""
        self.conn.commit()
        self.cursor.close()
        self.conn.close()

    def get_conn(self):
        """获取连接"""
        return self.conn

    def get_cursor(self):
        """获取游标"""
        return self.cursor

    def execute(self, sql, option=OPTION.select, d_list=None):
        """获取SQL执行结果：含增删改查"""
        if option == OPTION.select:
            self.cursor.execute(sql)
            return self.cursor.fetchall()
        elif option == OPTION.insertMany:
            return self.cursor.executemany(sql, d_list)
        else:
            return self.cursor.execute(sql)

    @staticmethod
    def execute_query_pd(sql):
        """获取SQL查询结果，pandas方式"""
        return pd.read_sql(sql, create_engine(f"mysql+pymysql://{DEFAULT['user']}:{DEFAULT['password']}@{DEFAULT['host']}:"
                                              f"{DEFAULT['port']}/{DATABASES['default_database']}"))



import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LinearRegression
import joblib
import os
from pywong.db.postgresql.utils import DbHelper
import warnings
import platform

# 忽略警告
warnings.filterwarnings('ignore')

"""
工作原理：
1）最优化损失函数，转换为求函数极值问题，即在导数为0处能够取得极大值或者极小值；
1）根据训练样本计算截距和系数；
3）评价指标：R2，越大越好，最大为1；小于0，表示可能不存在线性关系；
4）不需要对样本数据进行归一化处理。
"""


class MLinearRegression:
    """线性回归器"""

    def __init__(self, name, path='D:\\model'):
        """构造函数"""
        # 系数
        self.coef_ = None
        # 截距
        self.intercept_ = None
        self.X_train = None
        self.y_train = None
        self.X_test = None
        self.y_test = None
        self.model = LinearRegression()
        # 模型名称
        self.name = name
        self.model_path = self.get_model_path(path)

    @staticmethod
    def feature(path):
        """私有方法，返回数据，仅内部使用"""
        data_all = pd.read_csv(path)
        feature = data_all.columns.values.tolist()
        return data_all, feature

    def fit(self, target, path=None, sql=None, data=None, test_size=0.2, random_state=666):
        """拟合数据"""
        data_all = None
        feature = None
        if path is not None:
            data_all, feature = self.feature(path)
            feature.remove(target)

        if sql is not None:
            data_all = DbHelper.execute_query_pd(sql)
            feature = data_all.columns.values.tolist()
            feature.remove(target)

        if data is not None:
            data_all = data
            feature = data_all.columns.values.tolist()
            feature.remove(target)

        # 切分训练集和测试集
        self.X_train, self.X_test, self.y_train, self.y_test = train_test_split(np.array(data_all[feature]),
                                                                                np.array(data_all[[target]]),
                                                                                test_size=test_size,
                                                                                random_state=random_state)

        self.model.fit(self.X_train, self.y_train.reshape(-1, ))

        self.coef_ = self.model.coef_
        self.intercept_ = self.model.intercept_

        return self

    def predict_inner(self):
        """预测"""
        return self.model.predict(self.X_test)

    def predict_outer(self, path=None, sql=None, data=None):
        """预测外部输入"""
        data_all = None
        X_predict = None
        if path is not None:
            data_all, X_predict = self.feature(path)

        if sql is not None:
            data_all = DbHelper.execute_query_pd(sql)
            X_predict = data_all.columns.values.tolist()

        if data is not None:
            data_all = data
            X_predict = data_all.columns.values.tolist()

        return self.model.predict(np.array(data_all[X_predict]))

    def score(self):
        """评估模型准确率"""
        return self.model.score(self.X_test, self.y_test.reshape(-1, ))

    def get_model_path(self, path):
        """返回模型路径"""
        file = None
        if not os.path.exists(path):
            os.mkdir(path)

        if 'Windows' == platform.system():
            file = f"{path}\\{self.name}.model"

        if 'Linux' == platform.system():
            file = f"{path}/{self.name}.model"

        return os.path.abspath(file)

    def save(self):
        """保存模型"""
        joblib.dump(self.model, self.model_path)
        print(f'保存{self.model_path}.')

    def load(self):
        """加载模型"""
        self.model = joblib.load(self.model_path)
        print(f"加载{self.name}.model.")

    def __str__(self):
        return "MLinearRegression()"



class OPTION:
    select = "SELECT"
    insert = "INSERT"
    insertMany = "ISM"
    update = "UPDATE"
    delete = "DELETE"


DEFAULT = {
    "host": "127.0.0.1",
    "port": 5432,
    "user": "postgres",
    "password": "123456"
}

DATABASES = {
    "default_database": "test",
    "data_database": "demo"
}



from distutils.core import setup
from setuptools import find_packages

with open('README.md', 'r', encoding='UTF-8') as f:
    long_description = f.read()

setup(
    name='pywong',
    version='3.1.1',
    description='pywong工具模块.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='Apache-2.0 license',
    url='https://blog.csdn.net/qq_16284479?type=blog',
    packages=find_packages(),
    python_requires='>=3.8',
    platforms=['all'],
    data_files=['README.md'],
    author='ghy',
    author_email='1563713769@qq.com',
    classifiers=[
        "Programming Language :: Python :: 3",
        # 开发的目标用户
        'Intended Audience :: Developers'
    ],
    install_requires=['PyMySQL>=1.0.2',
                      # 'psycopg2>=2.9.3',
                      'psycopg2-binary>=2.9.3',
                      'pymssql>=2.2.5',
                      'pandas>=1.4.3',
                      'SQLAlchemy>=1.4.39',
                      'psutil>=5.9.1',
                      'scikit-learn>=1.1.1',
                      'joblib>=1.1.0',
                      'prophet>1.1'],
    py_modules=['pywong.db.settings',
                'pywong.db.mssql.settings', 'pywong.db.mssql.utils',
                'pywong.db.mysql.settings', 'pywong.db.mysql.utils',
                'pywong.db.postgresql.settings', 'pywong.db.postgresql.utils',
                'pywong.monitor.resource_monitor',
                'pywong.ml.sk.knn', 'pywong.ml.sk.linear_regression',
                'pywong.ml.sk.random_forest', 'pywong.ml.sk.time_series']
)
